<?php

/*******************************************
* HELPER CLASSSES
* The JSON API server is expected to return data as a JSON formatted string.
* In this example, we create classes with properties that match the expected JSON format.
* Each class implements the JsonSerializable interface and contains a jsonSerialize function, which is used 
* by PHP's json_encode. 
* Our API methods below will return instances of these classes, which will then be converted to JSON 
* and returned using the json_encode call in the main section below.
* NOTE THAT THIS IS ONLY ONE WAY OF ACCOMPLISHING THIS TASK. You can feel free to instead construct 
* and return properly formatted JSON strings if you wish, or use a library or any other method of returning JSON results.
*/

// SearchResults class encapsulates the result of a search for Vitrium
// This sample JSON Service can only search for Readers. 
// This class is used as a return value of the GetAllReaders function, which Vitrium uses to display readers (users) 
// in the Users tab of its Admin UI, and also to process analytics information. While the GetAllReaders endpoint 
// is not strictly required for SSO functionality, its implementation is still recommended. 
class SearchResults implements JsonSerializable {
    public $totalRecords;   // Total records found by the search; integer
    public $results;        // Array of results: Reader objects

    function __construct() {
    	$this->totalRecords = 0;
    	$this->results = [];
    }

    public function jsonSerialize() {
        return [
            'TotalRecords' => $this->totalRecords,
            'Results' => $this->results
        ];
    }
}

// Class encapsulating information about a page of data
// In the context of this sample implementation, the paged search parameters are only used 
// as a parameter to the GetAllReaders function.
// There is no jsonSerialize method here since objects of this class are not returned by this server,
// only parsed as an incoming parameter of the GetAllReaders function. The parameter is supplied by the Vitrium
// server, and will conform to the following format:
// {
// 	"index": 1,
//	"size": 20
// }
class DataPage {
	public $index;   // Integer page number
	public $size;    // Number of items per page

	// Construct a class instance from an object parsed from the incoming JSON
	// Note that if incoming JSON is not supplied, default values of 1 for page index and 20 for page size are used.
	function __construct($object = null) {
		$this->index = (isset($object) && property_exists($object, 'index')) ? $object->index : 1;
		$this->size = (isset($object) && property_exists($object, 'size')) ? $object->size : 20;
	}
}

// Class encapsulating information about data filter
// As with DataPage, this class is only used as a parameter to the GetAllReaders function.
// Again there is no jsonSerialize method since objects of this class are only parsed 
// as an incoming parameter of the GetAllReaders function. 
// The parameter is supplied by the Vitrium server, and although it is extendable, when used by Vitrium Analytics
// it will  conform to the following format:
// {
// 	"ids": ["userId1", "userId2", "userId3"]
// }
class DataFilter {
	public $ids;   // String array of reader IDs

	// Construct a class instance from an object parsed from the incoming JSON
	function __construct($object = null) {
		$this->ids = (isset($object) && property_exists($object, 'ids')) ? $object->ids : [];
	}
}

// Reader class encapsulates a Vitrium reader or user - someone trying to access a protected document
class Reader implements JsonSerializable {
	public $id;            // String uniquely identifying this reader/user. This can be any string.
	                       // It might be a GUID or an integer or anything else.
	                       // (note that Vitrium expects a string ID, so convert integers to strings, i.e. "123" instead of 123)
	public $username;      // String username. This can be an email or a custom username
	public $isActive;      // Boolean. This will normally be true; inactive users will be denied access to documents.
	public $accessPolicy;  // Object of type AccessPolicy, specifying the reader/user's access to documents.
	                       // Note that AccessPolicy is also part of the Permission class.
	                       // It can either be specified as part of a Reader, or can be returned as part of a Permission by the Authorize endpoint.
                           // If an AccessPolicy is specified for both a Reader object and a Permission object for the user,
	                       // the most lenient combination of AccessPolicy settings will be used (see AccessPolicy class comments)
	
    public function jsonSerialize() {
        return [
            'Id' => $this->id,
            'Username' => $this->username,
            'IsActive' => $this->isActive,
            'AccessPolicy' => $this->accessPolicy
        ];
    }
}

// AccessPolicy class encapsulates a Vitrium user's access settings that decide whether they have access to protected content.
// Value of null in any of these fields will ignore that policy setting.
// Note that an AccessPolicy can be specified as part of a Reader class or part of a Permission class.
// If separate AccessPolicies are specified, Vitrium will create a most lenient combination of the policies in an attempt
// to grant the user access to the content. For example, if one policy specifies a $documentLimit of 1 and the other 
// specifies a $documentLimit of 2, Vitrium will use the higher limit of 2. This goes for all the other policy settings as well.
class AccessPolicy implements JsonSerializable {
	public $pdfLimit;      // These 3 integer properties determine how many different devices can be used to access a particular document.
	public $browserLimit;  // You can either separately specify pdfLimit (number of devices on which a downloaded document can be opened)
	public $computersMax;  // combined with browserLimit (number of browsers which can be used to view a document online), in which case
	                       // computersMax should be set to -1. Alternately, you can set pdfLimit and browserLimit to -1 and specify 
	                       // a value for computersMax, which combines PDF and web document access. 

	public $expiryDate;            // A date value specifying when a user's access to a given document expires. 
	public $relativeExpiryInDays;  // Integer value specifying the number of days after first unlock after which access to the document expires.
	public $ipAddressesMax;     // Integer maximum number of different IP addresses that can unlock a web document
	public $ignoredIpAddresses; // String range of IP addresses that can unlock a document without counting towards the IpAddressesMax limit (eg. 1.0.0.1-1.0.0.2,1.0.0.5) 
	public $documentLimit;  // Integer number of different documents that the user can unlock
	public $openLimit;   // Integer number of times that this user can unlock this document
	public $printLimit;  // Integer number of times that the user can print this PDF document or use the Web Viewer Download To Print function
	public $webPrintLimit;  // Integer number of times that the user can print this document online

    public function jsonSerialize() {
        return [
            'PdfLimit' => $this->pdfLimit,
            'BrowserLimit' => $this->browserLimit,
            'ComputersMax' => $this->computersMax,
            'Expiry' => $this->expiryDate,
            'RelativeExpiryInDays' => $this->relativeExpiryInDays,
            'IpAddressesMax' => $this->ipAddressesMax,
            'IgnoredIpAddresses' => $this->ignoredIpAddresses,
            'DocumentLimit' => $this->documentLimit,
            'OpenLimit' => $this->openLimit,
            'PrintLimit' => $this->printLimit,
            'WebPrintLimit' => $this->webPrintLimit
        ];
    }
}

// This class encapsulates the Authenticate API parameter.
// This class is used as a parameter to the Authenticate function.
// There is no jsonSerialize method since objects of this class are not returned, only parsed as parameters. 
// The parameter is supplied by the Vitrium server, and will conform to the following format:
// {
// 	"UserName": "name@domain.com",
//	"Password": "Test123"
// }
class AuthenticateParameters {
	public $username;
	public $password;

	// Construct a class instance from an object parsed from the incoming JSON
	function __construct($object) {
		if (property_exists($object, 'UserName'))
			$this->username = $object->UserName;
		if (property_exists($object, 'Password'))
			$this->password = $object->Password;
	}
}

// Class encapsulating the results of Authenticate API; simply returns succeed: true or false 
class AuthenticateResult implements JsonSerializable {
	public $succeed;

	function __construct($success) {
		$this->succeed = $success;
	}

    public function jsonSerialize() {
    	return [ 'Succeed' => isset($this->succeed) && $this->succeed ];
    }	
}

// This class encapsulates the Authorize API parameter.
// This class is used as a parameter to the Authorize function.
// There is no jsonSerialize method since objects of this class are not returned, only parsed as parameters. 
// The parameter is supplied by the Vitrium server, and will conform to the following format:
// {
// 	"ReaderId": "1",   -- This is a string corresponding to Reader->id
//	"DocCode": "0000-1111-22222-33333333"   -- This is a Doc Code of Vitrium uploaded protected content
//  "DocId": "c91a3156-e1a9-4735-b18b-ce6733a6754c",   -- This is a unique GUID ID of Vitrium uploaded protected content
//  "DocExternalKey": "DOC123",   -- This is an external key associated with Vitrium uploaded protected content, and can be used as document ID in your system 
//  "ClientDeviceId": "{1487461123171-c97f1220-4875-938b-a5ca-db66796f3c2b}"   -- Device, or tracking ID, from which the content is being accessed
// }
// Note that although all this information will be provided by Vitrium, it is up to you to choose what information you use to authorize document access.
class AuthorizeParameters {
	public $readerId;
	public $docCode;
	public $docId;
	public $docExternalKey;
	public $clientDeviceId;

	// Construct a class instance from an object parsed from the incoming JSON
	function __construct($object) {
		if (property_exists($object, 'ReaderId'))
			$this->readerId = $object->ReaderId;
		if (property_exists($object, 'DocCode'))
			$this->docCode = $object->DocCode;
		if (property_exists($object, 'DocId'))
			$this->docId = $object->DocId;
		if (property_exists($object, 'DocExternalKey'))
			$this->docExternalKey = $object->DocExternalKey;
		if (property_exists($object, 'ClientDeviceId'))
			$this->clientDeviceId = $object->ClientDeviceId;
	}
}

// Class encapsulating the results of Authorize API 
class AuthorizeResult implements JsonSerializable {
	public $succeed;
	public $accessPolicy;

    public function jsonSerialize() {
    	return [ 
    		'Succeed' => isset($this->succeed) && $this->succeed,
    		'AccessPolicy' => $this->accessPolicy
    	];
    }	
}


// ******************************************
// API IMPLEMENTATION METHODS

/**
* This API endpoint is only used when a user is logging into a document manually. It is not used by SSO.
* It accepts POST requests and expects to receive JSON body in the following format:
* {
*   "UserName": "name@domain.com",
*   "Password": "Test123"
* }
*
* @return AuthenticateResult  The succeed property should be set to true/false 
*/
function Authenticate() {
	// Convert the JSON request body into a AuthenticateParameters object
	$authenticateParameters = new AuthenticateParameters(json_decode(file_get_contents('php://input')));

	// ### TODO: VALIDATION OF USERNAME AND PASSWORD NEEDS TO BE IMPLEMENTED HERE.

	// This simple example validates a user with username "name@domain.com" and password "Test123"
	return new AuthenticateResult($authenticateParameters->username == "name@domain.com" && $authenticateParameters->password == "Test123");
}

/**
* This method authorizes a customer to access a document.
* The request contains JSON body in the following format:
*   {
*     "ReaderId": "1",   -- string corresponding to Reader->id
*     "DocCode": "0000-1111-22222-33333333"   -- Doc Code of Vitrium uploaded protected content
*     "DocId": "c91a3156-e1a9-4735-b18b-ce6733a6754c",   -- unique GUID ID of Vitrium uploaded protected content
*     "DocExternalKey": "DOC123",   -- external key associated with Vitrium uploaded protected content, which can be used as document ID in your system 
*     "ClientDeviceId": "{1487461123171-c97f1220-4875-938b-a5ca-db66796f3c2b}"   -- Device, or tracking ID, from which the content is being accessed
*   }
* This JSON can be parsed into an AuthorizeParameter object.
*
* @return AuthorizeResult  The return value should be a AuthorizeResult object with the Succeeed property set to true or false 
*                          depending on the outcome of authorization, and AccessPolicy property set to the applicable AccessPolicy object
*                          or null to deny the customer access to the document.  
*/ 
function Authorize() {
	// Convert the JSON request body into an AuthorizeParameters object
	$authorizeParameters = new AuthorizeParameters(json_decode(file_get_contents('php://input')));

	// Initialize the result object. Initially Succeeed will be false and AccessPolicy will be null, representing authorization failure.
	$authorizeResult = new AuthorizeResult();

	// ### TODO: YOU NEED TO IMPLEMENT LOGIC TO DETERMINE WHAT LEVEL OF ACCESS THIS USER SHOULD HAVE TO THIS DOCUMENT.
	// ### At this point, the reader (user) will have passed authentication.
	// ### You can user the $authorizeParameters->readerId property to locate the user in your system.
	// ### $authorizeParameters->docCode and $authorizeParameters->docID properties contain Vitrium internal document 
	// ###   code and ID, which may or may not be useful to you.
	// ### $authorizeParameters->docExternalKey will contain an external key attached to the document, which you
	// ###   can configure when uploading documents to Vitrium. Normally, this would contain a document ID
	// ###   which will help you locate the document in your system, and is most likely the parameter you will use.
	// ### $authorizeParameters->clientDeviceId will contain a unique ID generated by Vitrium of the device that the user
	// ###   is using to access the document. It can be used to limit a user's access to a particular device or to a number
	// ###   of devices if necessary.

	// This simple example validates a user with ID "userId1" and gives the user unlimited access to document with external key "doc1"
    if ($authorizeParameters->readerId == 'userId1' && $authorizeParameters->docExternalKey == 'doc1') {
		$authorizeResult->succeed = true;
    	$authorizeResult->accessPolicy = new AccessPolicy();   // Default access policy sets all properties to null, representing an unlimited access policy.
    }
    else {
		$authorizeResult->succeed = false;
    	$authorizeResult->accessPolicy = null;
    }

	return $authorizeResult;
}

/**
* Retrieves readers (users) according to paging or filter criteria supplied 
* REQUIRES either a "page" or "filter" query string parameter; retrieving all readers without limitation is not supported
* @return array of Readers
* NOTE: Sorting is currently not supported.
*/
function GetAllReaders() {
	// Parse the page and/or filter parameters from the query string
	$rawPageQuery = isset($_GET["page"]) ? urldecode($_GET["page"]) : null;
	$rawFilterQuery = isset($_GET["filter"]) ? urldecode($_GET["filter"]) : null;

	// We either need paging parameters, or a filter with IDs. If we have neither, return empty result.
	if ((!isset($rawPageQuery) || empty($rawPageQuery))
		&& (!isset($rawFilterQuery) || empty($rawFilterQuery)))
		return $searchResult;

	// Parse the page and/or filter parameters into objects
	$page = (isset($rawPageQuery) && !empty($rawPageQuery)) ? new DataPage(json_decode($rawPageQuery)) : new DataPage();
	$filter = (isset($rawFilterQuery) && !empty($rawFilterQuery)) ? new DataFilter(json_decode($rawFilterQuery)) : new DataFilter();

	// ### TODO: IMPLEMENT RETRIEVING USER INFORMATION FROM YOUR SYSTEM USING THE PAGING AND/OR FILTERING PARAMETERS SUPPLIED
	// ### The results must be converted to Reader objects (or parsed into the expected JSON format another way).

	// In this simple example, the paging parameter is used to create an array or Reader objects with IDs "userIdX" 
	// and usernames "userNameX". You wouldn't be doing this in live implementation; this is merely an example of 
	// constructing a result array.
	$readerArray = [];
	if ($page->index > 0 && $page->size > 0) {
		$index = ($page->index - 1) * $page->size;
		for ($i = 0; $i < $page->size; $i++) {
			$reader = new Reader();
			$reader->id = "userId" . ++$index;
			$reader->username = "userName" . $index;
			$reader->isActive = true;

			// In this example, if a filter of IDs is supplied, only return those readers whose IDs match the filter values
			if (empty($filter->ids) || in_array($reader->id, $filter->ids))
				array_push($readerArray, $reader);
		}
	}

	// Construct and return search results expected by Vitrium
	$searchResult = new SearchResults();
	$searchResult->results = $readerArray;
	$searchResult->totalRecords = count($searchResult->results);
	return $searchResult;
}

/**
* Retrieves a reader (user) by ID
* @param int $id  Reader ID
* @return Reader object if found; otherwise sets HTTP status to 404 Not Found and returns null 
*/
function GetReaderById($id) {
	// ### TODO: RETRIEVE A USER BY ID FROM INTERNAL SYSTEM.
	// ### Note that the ID can be whatever you use internally to identify a user

	// In this simple example, if the ID starts with the string "userId", we construct and return a Reader object,
	// simulating a situation where the user was found in your system. If the ID starts with anything else, we simulate
	// a situation where the user was not found
	$reader = null;
	if (startsWith($id, "userId")) {
		$reader = new Reader();
		$reader->id = $id;
		$reader->username = "usernameExample";
		$reader->isActive = true;
	}

	// If a user was not found, set the response code to 404 (NotFound) and return a null value
	if (!isset($reader)) {
		http_response_code(404);
		return null;
	}

	// If the user was found, return their information
	return $reader;
}

/**
* Retrieves a reader(user) by username
* @param string $username  Reader username
* @return Reader object if found; otherwise response HTTP status should be 404 Not Found and return value should be null 
*/
function GetReaderByUsername($username) {
	// ### TODO: RETRIEVE A USER BY USERNAME FROM INTERNAL SYSTEM.

	// In this simple example, if the username starts with the string "userName", we construct and return a Reader object,
	// simulating a situation where the user was found in your system. If the username starts with anything else, we simulate
	// a situation where the user was not found
	$reader = null;
	if (startsWith($username, "userName")) {
		$reader = new Reader();
		$reader->id = "idExample";
		$reader->username = $username;
		$reader->isActive = true;
	}

	// If a user was not found, set the response code to 404 (NotFound) and return a null value
	if (!isset($reader)) {
		http_response_code(404);
		return null;
	}

	// If the user was found, return their information
	return $reader;
}

/**
* Retrieves a user by token. This method will be used for SSO.
* @param string $token  Hash created using the algorithm in the tokenHasher function
* @return Reader object if the user was found; otherwise response HTTP status should be 404 Not Found and return value should be null 
*/
function GetReaderByToken($token) {
	// ### TODO: RETRIEVE A USER BY TOKEN FROM INTERNAL SYSTEM.
	// ### The token can be any encrypted value. It can contain the user's ID or username or any other information
	// ### needed to identify the user in your system. In this example, we use a hashed value created using the tokenHasher 
	// ### function below. Note that this is the token used by Vitrium to retrieve a user from your system when this token
	// ### is included in the URL accessing a Vitrium document in Web Viewer. What you would need to do is create this token 
	// ### in your system, and include it in the link to access a document, for example:
	// ###   https://view.protectedpdf.com/aBcDeF&token=V3o2Z0JLYm9BMUxwNDd5enZTalc4Zz09
	// ### where "aBcDeF" is the Vitrium document alias, "token" is the name of the token parameter as specified in your
	// ### Vitrium SSO configuration, and "V3o2Z0JLYm9BMUxwNDd5enZTalc4Zz09" is the encrypted value "userId1". Note that 
	// ### the method used to hash the token is important since you need to be able to unhash it as well.

	// In this simple example, we will unhash the token and defer the retrieval of the user to the GetReaderById function.
	// Remember that GetReaderById example code will return a valid Reader object if the ID starts with "UserId"

	// Unhash the token; we're expecting a reader ID in this example, but this can be anything you need it to be.
	$readerId = tokenHasher('decrypt', urldecode($token));
	return GetReaderById($readerId);
}


/**
* Function to encrypt/decrypt a token.
* IMPORTANT: THE TOKEN MUST BE ENCRYPTED AND DECRYPTED USING THE SAME ALGORITHM.
* This is an example implementation here; you are free to implement your own.
* Adapted from https://andrewho.nl/encrypt-decrypt-strings-php-easy-way/
* @param string $action  This should be 'encrypt' or 'decrypt'
* @param string $string  String to encrypt or decrypt
* @return string $output Encrypted or decrypted string
*/ 
function tokenHasher($action, $string) {
    $output = false;
 
    $encrypt_method = "AES-256-CBC";
    $secret_key = '$ecr3tKey80ed0a2c-c9c2-42a7-b8b0-3a9d64d5f0b7';
    $secret_iv = 'In1tV3@t0r712da4ee-dc2f-499a-b5d6-bea1d1658106';   // Initialization vector
 
    // hash the secret key
    $key = hash('sha256', $secret_key);
    
    // hash the initialization vector - encrypt method AES-256-CBC expects 16 bytes
    $iv = substr(hash('sha256', $secret_iv), 0, 16);
 
    if( $action == 'encrypt' ) {
        $output = openssl_encrypt($string, $encrypt_method, $key, 0, $iv);
        $output = base64_encode($output);
    }
    else if( $action == 'decrypt' ){
        $output = openssl_decrypt(base64_decode($string), $encrypt_method, $key, 0, $iv);
    }
 
    return $output;
}

/**
* Helper function to test token hashing in the browser by navigating to "/tokentest" endpoint
* @param string $queryString   Query string, which should be in the format action={encrypt|decrypt}&string={string to encrypt or decrypt}
* @return string $result       Hashed or unhashed string if query string is valid; else a readable error message
*/
function TestTokenHashing($queryString) {
	$errorPreamble = "TokenTest endpoint was called with incorrect parameters. " .
			"Expected parameters in format action={encrypt|decrypt}&string={string to encrypt or decrypt}";

	// Split the query string on ampersand; we should end up with 2 strings starting with 'action' and 'string'
	$tokenTestParameters = explode('&', $queryString);
	if (count($tokenTestParameters) != 2
		|| !Helpers::startsWith($tokenTestParameters[0], 'action')
		|| !Helpers::startsWith($tokenTestParameters[1], 'string'))
	{
		return $errorPreamble . " but received " . $queryString;
	}

	// Split the action parameter on equal sign; we should end up with strings 'action' and ('encrypt' or 'decrypt')
	$actionStrings = explode('=', $tokenTestParameters[0]);
	if (count($actionStrings) != 2
		|| ($actionStrings[1] != 'encrypt' && $actionStrings[1] != 'decrypt'))
	{
		return $errorPreamble . ". Received action parameter " . $tokenTestParameters[0];
	}

	$action = $actionStrings[1];   // At this point action is validated

	// Split the string parameter on equal sign; we should end up with 2 strings
	$stringStrings = explode('=', $tokenTestParameters[1]);
	if (count($stringStrings) != 2)
	{
		return $errorPreamble . ". Received string parameter " . $tokenTestParameters[1];
	}

	$string = urldecode($stringStrings[1]);
	$result = tokenHasher($action, $string);

	return [ 'Action' => $action, 'String' => $string, 'Result' => $result ];
}

/**
* Helper function that returs true if $haystack string starts with $needle 
*/
function startsWith($haystack, $needle) {
     return (substr($haystack, 0, strlen($needle)) === $needle);
}

/**
* Helper function that returs true if $haystack string ends with $needle 
*/
function endsWith($haystack, $needle) {
     return (substr($haystack, strlen($needle) * -1) === $needle);
}

/**
* Generates a new GUID.
* Adapted from https://stackoverflow.com/questions/21671179/how-to-generate-a-new-guid
*/
function createGuid() {
    if (function_exists('com_create_guid'))
        return trim(com_create_guid(), '{}');

    return sprintf('%04X%04X-%04X-%04X-%04X-%04X%04X%04X', mt_rand(0, 65535), mt_rand(0, 65535), mt_rand(0, 65535), mt_rand(16384, 20479), mt_rand(32768, 49151), mt_rand(0, 65535), mt_rand(0, 65535), mt_rand(0, 65535));
}


// **************************************** MAIN SECTION
// Examine the URI and run the appropriate method.
// This is extremely simple routing; many more robust options and available
// but since we are only implementing a very small, specific set of URIs,
// we can get away with this simple implementation
// Note that most API methods expect to receive parameters in a query string.
// Here we parse those parameters out from query string and supply them as parameters to function calls.
$uri = $_SERVER['REQUEST_URI'];
if (startsWith($uri, '/authenticate'))
	$result = Authenticate();
elseif (startsWith($uri, '/authorize'))
	$result = Authorize();
elseif (startsWith($uri, '/readers'))
	$result = GetAllReaders();
elseif (startsWith($uri, '/reader')) {
	$readerParameters = substr($uri, strlen('/reader'));
	if (startsWith($readerParameters, '/'))
		$result = GetReaderById(substr($readerParameters, 1));
	// Note: $_GET could be used here instead of string parsing
	elseif (startsWith($readerParameters, '?username='))
		$result = GetReaderByUsername(substr($readerParameters, strlen('?username=')));
	elseif (startsWith($readerParameters, '?token='))
		$result = GetReaderByToken(substr($readerParameters, strlen('?token=')));
	else
		$result = 'Reader endpoint was called with incorrect parameters: ' . trim($readerParameters, '/?');
}
elseif (startsWith($uri, '/tokentest')) {
	$result = TestTokenHashing(substr($uri, strlen('/tokentest?')));
}
else $result = 'Endpoint ' . $uri . ' is not implemented.';

// Set the JSON response header and return the result
header('Content-type: application/json');
echo json_encode($result);   // RETURN THE RESULT IN JSON FORMAT

?>